/*
   occ.h: interface Orthogonal Collocation class.
   ---------------------------------------------------
   This Class supports Gauss or Lobatto base points
   Nonsymmetric problems in Planar geometry
   Symmetric problems in Planar, Cylindrical or Spherical geometry

   There are no hardwired dimensions, so any number of points
   may be specified.

   CollocSet must be called first to specify
       Symmetry, Type (Lobatto or Gauss) and Geometry
       and to set the Jacobi weight factors:
          ((1 - x)**alpha)*(x**beta)

   The arrays may be obtained individually using the array valued
   funcions: Xpoints, Weights, Acoeff, Bcoeff, CCoeff or MassMat
   Or CollocCoef can be called to get X, W, A and optionally
   B and C all at once.

   The Jacobi & QWeights codes are general for other Jacobi
   polynomials weights.

   The precision can be changed by changing parameter "float"
   --------------------------------------------------- */

#if !defined(OCC_INCLUDED)
#define OCC_INCLUDED
#include <stdio.h>
#include "array.h"
class Orthcc{
public:
   Orthcc(){}
   Orthcc(int s, int t, int g) : sym(s), typ(t), igeom(g) {}
   virtual ~Orthcc();
   static const int Nonsymmetric=0;
   static const int Symmetric=1;
   static const int Planar=0;
   static const int Cylindrical=1;
   static const int Spherical=2;
   static const int Gauss=1;
   static const int Lobatto=2;
   void occ_set(int s, int t, int g);
   void Quadrature(double *x, double *w, int n);
   void Acoeff(Array2D &A, double *x, int n);
   void Anonsym(Array2D &An, Array2D &A, double *x);
   void Bcoeff(Array2D &B, Array2D &A, double *x);
   void Ccoeff(Array2D &C, Array2D &A, double *x, double *w);
   void MassMat(Array2D &D, double *x, double *w, int n);
   void Lcoeff(double *L, double *x, double xi, int n);
   void LcoeffN(Array2D &L, double *x, double *xi, int n, int ni);
private:
   int sym;    // 0,1 nonsymmetric, symmetric
   int typ;    // 1,2 Gauss, Lobatto
   int igeom;  // 0,1,2 planar, cylindrical, spherical
   void MassMatGM(double *x,int n, Array2D &D);
   void MassMatMM(double *x,int n, Array2D &D);
   int status;
   void cdgqf ( int nt, int kind, double alpha, double beta, double t[],double wts[] );
   void cgqf ( int nt, int kind, double alpha, double beta, double a, double b,double t[], double wts[] );
   double class_matrix ( int kind, int m, double alpha, double beta, double aj[],double bj[] );
   void imtqlx ( int n, double d[], double e[], double z[] );
   void parchk ( int kind, int m, double alpha, double beta );
   void scqf ( int nt, double t[], int mlt[], double wts[], int nwts, int ndx[],
      double swts[], double st[], int kind, double alpha, double beta, double a, double b );
   void sgqf ( int nt, double aj[], double bj[], double zemu, double t[], double wts[] );
};
#endif // !defined(OCC_INCLUDED)
